@@ -11,12 +11,13 @@ import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.introspect.Annotated;
+import com.fasterxml.jackson.databind.introspect.AnnotatedClass;
 import com.fasterxml.jackson.databind.introspect.ClassIntrospector;
+import com.fasterxml.jackson.databind.introspect.NopAnnotationIntrospector;
 import com.fasterxml.jackson.databind.introspect.VisibilityChecker;
 import com.fasterxml.jackson.databind.jsontype.SubtypeResolver;
 import com.fasterxml.jackson.databind.jsontype.TypeIdResolver;
 import com.fasterxml.jackson.databind.jsontype.TypeResolverBuilder;
-import com.fasterxml.jackson.databind.type.TypeBindings;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
@@ -36,7 +37,17 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     implements ClassIntrospector.MixInResolver,
         java.io.Serializable
 {
-    private static final long serialVersionUID = 8891625428805876137L;
+    private static final long serialVersionUID = 2L; // since 2.9
+
+    /**
+     * @since 2.7
+     */
+    protected final static JsonInclude.Value EMPTY_INCLUDE = JsonInclude.Value.empty();
+
+    /**
+     * @since 2.7
+     */
+    protected final static JsonFormat.Value EMPTY_FORMAT = JsonFormat.Value.empty();
 
     /**
      * Set of shared mapper features enabled.
@@ -47,7 +58,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * Immutable container object for simple configuration settings.
      */
     protected final BaseSettings _base;
-    
+
     /*
     /**********************************************************
     /* Life-cycle: constructors
@@ -60,6 +71,18 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
         _mapperFeatures = mapperFeatures;
     }
 
+    protected MapperConfig(MapperConfig<T> src, int mapperFeatures)
+    {
+        _base = src._base;
+        _mapperFeatures = mapperFeatures;
+    }
+
+    protected MapperConfig(MapperConfig<T> src, BaseSettings base)
+    {
+        _base = base;
+        _mapperFeatures = src._mapperFeatures;
+    }
+    
     protected MapperConfig(MapperConfig<T> src)
     {
         _base = src._base;
@@ -210,22 +233,12 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * Non-final since it is actually overridden by sub-classes (for now?)
      */
     public AnnotationIntrospector getAnnotationIntrospector() {
-        return _base.getAnnotationIntrospector();
+        if (isEnabled(MapperFeature.USE_ANNOTATIONS)) {
+            return _base.getAnnotationIntrospector();
+        }
+        return NopAnnotationIntrospector.instance;
     }
 
-    /**
-     * Accessor for object used for determining whether specific property elements
-     * (method, constructors, fields) can be auto-detected based on
-     * their visibility (access modifiers). Can be changed to allow
-     * different minimum visibility levels for auto-detection. Note
-     * that this is the global handler; individual types (classes)
-     * can further override active checker used (using
-     * {@link JsonAutoDetect} annotation)
-     */
-    public VisibilityChecker<?> getDefaultVisibilityChecker() {
-        return _base.getVisibilityChecker();
-    }
-    
     public final PropertyNamingStrategy getPropertyNamingStrategy() {
         return _base.getPropertyNamingStrategy();
     }
@@ -233,7 +246,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     public final HandlerInstantiator getHandlerInstantiator() {
         return _base.getHandlerInstantiator();
     }
-    
+
     /*
     /**********************************************************
     /* Configuration: type and subtype handling
@@ -265,7 +278,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      *</pre>
      */
     public final JavaType constructType(Class<?> cls) {
-        return getTypeFactory().constructType(cls, (TypeBindings) null);
+        return getTypeFactory().constructType(cls);
     }
 
     /**
@@ -277,13 +290,13 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      *</pre>
      */
     public final JavaType constructType(TypeReference<?> valueTypeRef) {
-        return getTypeFactory().constructType(valueTypeRef.getType(), (TypeBindings) null);
+        return getTypeFactory().constructType(valueTypeRef.getType());
     }
 
     public JavaType constructSpecializedType(JavaType baseType, Class<?> subclass) {
         return getTypeFactory().constructSpecializedType(baseType, subclass);
     }
-    
+
     /*
     /**********************************************************
     /* Configuration: introspection support
@@ -297,12 +310,14 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     public BeanDescription introspectClassAnnotations(Class<?> cls) {
         return introspectClassAnnotations(constructType(cls));
     }
-    
+
     /**
      * Accessor for getting bean description that only contains class
      * annotations: useful if no getter/setter/creator information is needed.
      */
-    public abstract BeanDescription introspectClassAnnotations(JavaType type);
+    public BeanDescription introspectClassAnnotations(JavaType type) {
+        return getClassIntrospector().forClassAnnotations(this, type, this);
+    }
 
     /**
      * Accessor for getting bean description that only contains immediate class
@@ -312,12 +327,206 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     public BeanDescription introspectDirectClassAnnotations(Class<?> cls) {
         return introspectDirectClassAnnotations(constructType(cls));
     }
+
     /**
      * Accessor for getting bean description that only contains immediate class
      * annotations: ones from the class, and its direct mix-in, if any, but
      * not from super types.
      */
-    public abstract BeanDescription introspectDirectClassAnnotations(JavaType type);
+    public final BeanDescription introspectDirectClassAnnotations(JavaType type) {
+        return getClassIntrospector().forDirectClassAnnotations(this, type, this);
+    }
+
+    /*
+    /**********************************************************
+    /* Configuration: default settings with per-type overrides
+    /**********************************************************
+     */
+
+    /**
+     * Accessor for finding {@link ConfigOverride} to use for
+     * properties of given type, if any exist; or return `null` if not.
+     *<p>
+     * Note that only directly associated override
+     * is found; no type hierarchy traversal is performed.
+     *
+     * @since 2.8
+     * 
+     * @return Override object to use for the type, if defined; null if none.
+     */
+    public abstract ConfigOverride findConfigOverride(Class<?> type);
+
+    /**
+     * Accessor for finding {@link ConfigOverride} to use for
+     * properties of given type, if any exist; or if none, return an immutable
+     * "empty" instance with no overrides.
+     *<p>
+     * Note that only directly associated override
+     * is found; no type hierarchy traversal is performed.
+     *
+     * @since 2.9
+     * 
+     * @return Override object to use for the type, never null (but may be empty)
+     */
+    public abstract ConfigOverride getConfigOverride(Class<?> type);
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * used unless overridden by per-type or per-property overrides.
+     *
+     * @since 2.7
+     */
+    public abstract JsonInclude.Value getDefaultPropertyInclusion();
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * considering possible per-type override for given base type.<br>
+     * NOTE: if no override found, defaults to value returned by
+     * {@link #getDefaultPropertyInclusion()}.
+     *
+     * @since 2.7
+     */
+    public abstract JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType);
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * considering possible per-type override for given base type; but
+     * if none found, returning given <code>defaultIncl</code>
+     *
+     * @param defaultIncl Inclusion setting to return if no overrides found.
+     * 
+     * @since 2.8.2
+     */
+    public JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType,
+            JsonInclude.Value defaultIncl)
+    {
+        JsonInclude.Value v = getConfigOverride(baseType).getInclude();
+        if (v != null) {
+            return v;
+        }
+        return defaultIncl;
+    }
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * considering possible per-type override for given base type and
+     * possible per-type override for given property type.<br>
+     * NOTE: if no override found, defaults to value returned by
+     * {@link #getDefaultPropertyInclusion()}.
+     *
+     * @param baseType Type of the instance containing the targeted property.
+     * @param propertyType Type of the property to look up inclusion setting for.
+     *
+     * @since 2.9
+     */
+    public abstract JsonInclude.Value getDefaultInclusion(Class<?> baseType,
+            Class<?> propertyType);
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * considering possible per-type override for given base type and
+     * possible per-type override for given property type; but
+     * if none found, returning given <code>defaultIncl</code>
+     *
+     * @param baseType Type of the instance containing the targeted property.
+     * @param propertyType Type of the property to look up inclusion setting for.
+     * @param defaultIncl Inclusion setting to return if no overrides found.
+     *
+     * @since 2.9
+     */
+    public JsonInclude.Value getDefaultInclusion(Class<?> baseType,
+            Class<?> propertyType, JsonInclude.Value defaultIncl)
+    {
+        JsonInclude.Value baseOverride = getConfigOverride(baseType).getInclude();
+        JsonInclude.Value propOverride = getConfigOverride(propertyType).getIncludeAsProperty();
+
+        JsonInclude.Value result = JsonInclude.Value.mergeAll(defaultIncl, baseOverride, propOverride);
+        return result;
+    }
+
+    /**
+     * Accessor for default format settings to use for serialization (and, to a degree
+     * deserialization), considering baseline settings and per-type defaults
+     * for given base type (if any).
+     *
+     * @since 2.7
+     */
+    public abstract JsonFormat.Value getDefaultPropertyFormat(Class<?> baseType);
+
+    /**
+     * Accessor for default property ignorals to use, if any, for given base type,
+     * based on config overrides settings (see {@link #findConfigOverride(Class)}).
+     *
+     * @since 2.8
+     */
+    public abstract JsonIgnoreProperties.Value getDefaultPropertyIgnorals(Class<?> baseType);
+
+    /**
+     * Helper method that may be called to see if there are property ignoral
+     * definitions from annotations (via {@link AnnotatedClass}) or through
+     * "config overrides". If both exist, config overrides have precedence
+     * over class annotations.
+     *
+     * @since 2.8
+     */
+    public abstract JsonIgnoreProperties.Value getDefaultPropertyIgnorals(Class<?> baseType,
+            AnnotatedClass actualClass);
+
+    /**
+     * Accessor for object used for determining whether specific property elements
+     * (method, constructors, fields) can be auto-detected based on
+     * their visibility (access modifiers). Can be changed to allow
+     * different minimum visibility levels for auto-detection. Note
+     * that this is the global handler; individual types (classes)
+     * can further override active checker used (using
+     * {@link JsonAutoDetect} annotation)
+     */
+    public abstract VisibilityChecker<?> getDefaultVisibilityChecker();
+
+    /**
+     * Accessor for object used for determining whether specific property elements
+     * (method, constructors, fields) can be auto-detected based on
+     * their visibility (access modifiers). This is based on global defaults
+     * (as would be returned by {@link #getDefaultVisibilityChecker()}, but
+     * then modified by possible class annotation (see {@link JsonAutoDetect})
+     * and/or per-type config override (see {@link ConfigOverride#getVisibility()}).
+     *
+     * @since 2.9
+     */
+    public abstract VisibilityChecker<?> getDefaultVisibilityChecker(Class<?> baseType,
+            AnnotatedClass actualClass);
+
+    /**
+     * Accessor for the baseline setter info used as the global baseline,
+     * not considering possible per-type overrides.
+     *
+     * @return Global base settings; never null
+     *
+     * @since 2.9
+     */
+    public abstract JsonSetter.Value getDefaultSetterInfo();
+
+    /**
+     * Accessor for the baseline merge info used as the global baseline,
+     * not considering possible per-type overrides.
+     *
+     * @return Global base settings, if any; `null` if none.
+     *
+     * @since 2.9
+     */
+    public abstract Boolean getDefaultMergeable();
+
+    /**
+     * Accessor for the baseline merge info used for given type, including global
+     * defaults if no type-specific overrides defined.
+     *
+     * @return Type-specific settings (if any); global defaults (same as
+     *    {@link #getDefaultMergeable()}) otherwise, if any defined; or `null`
+     *    if neither defined
+     *
+     * @since 2.9
+     */
+    public abstract Boolean getDefaultMergeable(Class<?> baseType);
 
     /*
     /**********************************************************
@@ -382,6 +591,16 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      */
     public abstract ContextAttributes getAttributes();
 
+    /**
+     * @since 2.6
+     */
+    public abstract PropertyName findRootName(JavaType rootType);
+
+    /**
+     * @since 2.6
+     */
+    public abstract PropertyName findRootName(Class<?> rawRootType);
+
     /*
     /**********************************************************
     /* Methods for instantiating handlers
